@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: DataStream.java 1761021 2016-09-16 11:40:57Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.afp;
 
@@ -485,7 +485,6 @@ public class DataStream {
      */
     public void createIncludePageOverlay(String name, int x, int y) {
         currentPageObject.createIncludePageOverlay(name, x, y, paintingState.getRotation());
-        currentPageObject.getActiveEnvironmentGroup().createOverlay(name);
     }
 
     /**
@@ -495,7 +494,13 @@ public class DataStream {
      *            the name of the medium map
      */
     public void createInvokeMediumMap(String name) {
-        currentPageGroup.createInvokeMediumMap(name);
+        if (currentPageGroup != null) {
+            currentPageGroup.createInvokeMediumMap(name);
+        } else if (currentPage != null) {
+            currentPage.createInvokeMediumMap(name);
+        } else {
+            document.createInvokeMediumMap(name);
+        }
     }
 
     /**
@@ -576,13 +581,10 @@ public class DataStream {
      * @param encoding The CCSID character set encoding
      */
     public void createTagLogicalElement(String name, String value, int encoding) {
-
         TagLogicalElement.State tleState = new  TagLogicalElement.State(name, value, encoding);
         if (currentPage != null) {
-
             currentPage.createTagLogicalElement(tleState);
-
-        } else {
+        } else if (currentPageGroup != null) {
             currentPageGroup.createTagLogicalElement(tleState);
         }
     }
