@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FopFactory.java 1884907 2020-12-29 13:33:34Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.apps;
 
@@ -24,6 +24,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.URI;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
@@ -142,8 +144,14 @@ public final class FopFactory implements ImageContext {
      * @param baseURI the base URI to resolve resource URIs against
      * @return the requested FopFactory instance.
      */
-    public static FopFactory newInstance(URI baseURI) {
-        return new FopFactoryBuilder(baseURI).build();
+    public static FopFactory newInstance(final URI baseURI) {
+        return AccessController.doPrivileged(
+            new PrivilegedAction<FopFactory>() {
+                public FopFactory run() {
+                    return new FopFactoryBuilder(baseURI).build();
+                }
+            }
+        );
     }
 
     /**
@@ -156,9 +164,26 @@ public final class FopFactory implements ImageContext {
      * @throws SAXException
      * @throws IOException
      */
-    public static FopFactory newInstance(URI baseURI, InputStream confStream) throws SAXException,
+    public static FopFactory newInstance(final URI baseURI, final InputStream confStream) throws SAXException,
             IOException {
-        return new FopConfParser(confStream, baseURI).getFopFactoryBuilder().build();
+        Object action = AccessController.doPrivileged(
+            new PrivilegedAction<Object>() {
+                public Object run() {
+                    try {
+                        return new FopConfParser(confStream, baseURI).getFopFactoryBuilder().build();
+                    } catch (SAXException | IOException e) {
+                        return e;
+                    }
+                }
+            }
+        );
+        if (action instanceof SAXException) {
+            throw (SAXException) action;
+        }
+        if (action instanceof IOException) {
+            throw (IOException) action;
+        }
+        return (FopFactory) action;
     }
 
     /**
@@ -211,6 +236,22 @@ public final class FopFactory implements ImageContext {
         return config.isTableBorderOverpaint();
     }
 
+    boolean isSimpleLineBreaking() {
+        return config.isSimpleLineBreaking();
+    }
+
+    boolean isSkipPagePositionOnlyAllowed() {
+        return config.isSkipPagePositionOnlyAllowed();
+    }
+
+    boolean isLegacySkipPagePositionOnly() {
+        return config.isLegacySkipPagePositionOnly();
+    }
+
+    boolean isLegacyLastPageChangeIPD() {
+        return config.isLegacyLastPageChangeIPD();
+    }
+
     /**
      * Returns a new {@link Fop} instance. FOP will be configured with a default user agent
      * instance. Use this factory method if your output type requires an output stream.
