@@ -26,6 +26,8 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 
+import org.xml.sax.SAXException;
+
 import org.apache.xmlgraphics.image.loader.ImageContext;
 import org.apache.xmlgraphics.image.loader.ImageManager;
 import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext.FallbackResolver;
@@ -301,7 +303,18 @@ public final class FopFactoryBuilder {
      * @return <code>this</code>
      */
     public FopFactoryBuilder setConfiguration(Configuration cfg) {
+        return setConfiguration(cfg, true);
+    }
+
+    protected FopFactoryBuilder setConfiguration(Configuration cfg, boolean parse) {
         fopFactoryConfigBuilder.setConfiguration(cfg);
+        if (parse) {
+            try {
+                new FopConfParser(cfg, this);
+            } catch (SAXException e) {
+                throw new RuntimeException(e);
+            }
+        }
         return this;
     }
 
@@ -332,6 +345,26 @@ public final class FopFactoryBuilder {
         return this;
     }
 
+    public FopFactoryBuilder setSimpleLineBreaking(boolean b) {
+        fopFactoryConfigBuilder.setSimpleLineBreaking(b);
+        return this;
+    }
+
+    public FopFactoryBuilder setSkipPagePositionOnlyAllowed(boolean b) {
+        fopFactoryConfigBuilder.setSkipPagePositionOnlyAllowed(b);
+        return this;
+    }
+
+    public FopFactoryBuilder setLegacySkipPagePositionOnly(boolean b) {
+        fopFactoryConfigBuilder.setLegacySkipPagePositionOnly(b);
+        return this;
+    }
+
+    public FopFactoryBuilder setLegacyLastPageChangeIPD(boolean b) {
+        fopFactoryConfigBuilder.setLegacyLastPageChangeIPD(b);
+        return this;
+    }
+
     public static class FopFactoryConfigImpl implements FopFactoryConfig {
 
         private final EnvironmentProfile enviro;
@@ -374,6 +407,13 @@ public final class FopFactoryBuilder {
         private Map<String, String> hyphPatNames;
 
         private boolean tableBorderOverpaint;
+        private boolean simpleLineBreaking;
+
+        private boolean skipPagePositionOnlyAllowed = true;
+
+        private boolean legacySkipPagePositionOnly;
+
+        private boolean legacyLastPageChangeIPD;
 
         private static final class ImageContextImpl implements ImageContext {
 
@@ -495,6 +535,22 @@ public final class FopFactoryBuilder {
             return tableBorderOverpaint;
         }
 
+        public boolean isSimpleLineBreaking() {
+            return simpleLineBreaking;
+        }
+
+        public boolean isSkipPagePositionOnlyAllowed() {
+            return skipPagePositionOnlyAllowed;
+        }
+
+        public boolean isLegacySkipPagePositionOnly() {
+            return legacySkipPagePositionOnly;
+        }
+
+        public boolean isLegacyLastPageChangeIPD() {
+            return legacyLastPageChangeIPD;
+        }
+
         public Map<String, String> getHyphenationPatternNames() {
             return hyphPatNames;
         }
@@ -542,6 +598,14 @@ public final class FopFactoryBuilder {
         void setHyphPatNames(Map<String, String> hyphPatNames);
 
         void setTableBorderOverpaint(boolean b);
+
+        void setSimpleLineBreaking(boolean b);
+
+        void setSkipPagePositionOnlyAllowed(boolean b);
+
+        void setLegacySkipPagePositionOnly(boolean b);
+
+        void setLegacyLastPageChangeIPD(boolean b);
     }
 
     private static final class CompletedFopFactoryConfigBuilder implements FopFactoryConfigBuilder {
@@ -629,6 +693,22 @@ public final class FopFactoryBuilder {
         public void setTableBorderOverpaint(boolean b) {
             throwIllegalStateException();
         }
+
+        public void setSimpleLineBreaking(boolean b) {
+            throwIllegalStateException();
+        }
+
+        public void setSkipPagePositionOnlyAllowed(boolean b) {
+            throwIllegalStateException();
+        }
+
+        public void setLegacySkipPagePositionOnly(boolean b) {
+            throwIllegalStateException();
+        }
+
+        public void setLegacyLastPageChangeIPD(boolean b) {
+            throwIllegalStateException();
+        }
     }
 
     private static final class ActiveFopFactoryConfigBuilder implements FopFactoryConfigBuilder {
@@ -717,6 +797,22 @@ public final class FopFactoryBuilder {
         public void setTableBorderOverpaint(boolean b) {
             config.tableBorderOverpaint = b;
         }
+
+        public void setSimpleLineBreaking(boolean b) {
+            config.simpleLineBreaking = b;
+        }
+
+        public void setSkipPagePositionOnlyAllowed(boolean b) {
+            config.skipPagePositionOnlyAllowed = b;
+        }
+
+        public void setLegacySkipPagePositionOnly(boolean b) {
+            config.legacySkipPagePositionOnly = b;
+        }
+
+        public void setLegacyLastPageChangeIPD(boolean b) {
+            config.legacyLastPageChangeIPD = b;
+        }
     }
 
 }
