@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FObj.java 1875645 2020-03-25 14:44:35Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.fo;
 
@@ -314,7 +314,7 @@ public abstract class FObj extends FONode implements Constants {
         }
         if (child == lastChild) {
             if (child.siblings != null) {
-                lastChild = siblings[0];
+                lastChild = child.siblings[0];
             } else {
                 lastChild = null;
             }
