@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BlockContainerLayoutManager.java 1877372 2020-05-05 08:12:02Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr;
 
@@ -182,7 +182,7 @@ public class BlockContainerLayoutManager extends SpacedBorderedPaddedBlockLayout
 
     /** {@inheritDoc} */
     @Override
-    public List getNextKnuthElements(LayoutContext context, int alignment) {
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
         return getNextKnuthElements(context, alignment, null, null, null);
     }
 
@@ -203,7 +203,7 @@ public class BlockContainerLayoutManager extends SpacedBorderedPaddedBlockLayout
 
     /** {@inheritDoc} */
     @Override
-    public List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
         Position restartPosition, LayoutManager restartAtLM) {
 
         resetSpaces();
@@ -659,14 +659,14 @@ public class BlockContainerLayoutManager extends SpacedBorderedPaddedBlockLayout
             return lc;
         }
 
-        protected List getNextKnuthElements(LayoutContext context, int alignment) {
+        protected List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
             LayoutManager curLM; // currently active LM
             List<ListElement> returnList = new LinkedList<ListElement>();
 
             while ((curLM = getChildLM()) != null) {
                 LayoutContext childLC = makeChildLayoutContext(context);
 
-                List returnedList = null;
+                List<ListElement> returnedList = null;
                 if (!curLM.isFinished()) {
                     returnedList = curLM.getNextKnuthElements(childLC, alignment);
                 }
