@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreaker.java 1880106 2020-07-21 13:28:42Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr;
 
@@ -56,8 +56,8 @@ public class PageBreaker extends AbstractBreaker {
     private int floatHeight;
     private int floatYOffset;
 
-    private List relayedFootnotesList;
-    private List relayedLengthList;
+    private List<ListElement> relayedFootnotesList;
+    private List<Integer> relayedLengthList;
     private int relayedTotalFootnotesLength;
     private int relayedInsertedFootnotesLength;
     private boolean relayedFootnotesPending;
@@ -161,7 +161,7 @@ public class PageBreaker extends AbstractBreaker {
 
     /** {@inheritDoc} */
     protected int getNextBlockList(LayoutContext childLC, int nextSequenceStartsOn,
-            Position positionAtIPDChange, LayoutManager restartLM, List firstElements) {
+            Position positionAtIPDChange, LayoutManager restartLM, List<ListElement> firstElements) {
         if (!layoutRedone && !handlingFloat()) {
             if (!firstPart) {
                 // if this is the first page that will be created by
@@ -180,7 +180,7 @@ public class PageBreaker extends AbstractBreaker {
                 restartLM, firstElements);
     }
 
-    private boolean containsFootnotes(List contentList, LayoutContext context) {
+    private boolean containsFootnotes(List<ListElement> contentList, LayoutContext context) {
         boolean containsFootnotes = false;
         if (contentList != null) {
             for (Object aContentList : contentList) {
@@ -201,9 +201,9 @@ public class PageBreaker extends AbstractBreaker {
         return containsFootnotes;
     }
 
-    public static  List<List<KnuthElement>> getFootnoteKnuthElements(FlowLayoutManager flowLM, LayoutContext context,
+    public static List<List<KnuthElement>> getFootnoteKnuthElements(FlowLayoutManager flowLM, LayoutContext context,
             List<FootnoteBodyLayoutManager> footnoteBodyLMs) {
-        List<List<KnuthElement>> footnotes = new ArrayList<List<KnuthElement>>();
+        List<List<KnuthElement>> footnotes = new ArrayList<>();
         LayoutContext footnoteContext = LayoutContext.copyOf(context);
         footnoteContext.setStackLimitBP(context.getStackLimitBP());
         footnoteContext.setRefIPD(flowLM.getPSLM()
@@ -245,8 +245,8 @@ public class PageBreaker extends AbstractBreaker {
     }
 
     /** {@inheritDoc} */
-    protected List getNextKnuthElements(LayoutContext context, int alignment) {
-        List contentList = null;
+    protected List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
+        List<ListElement> contentList = null;
 
         while (!childFLM.isFinished() && contentList == null) {
             contentList = childFLM.getNextKnuthElements(context, alignment);
@@ -262,9 +262,9 @@ public class PageBreaker extends AbstractBreaker {
     }
 
     /** {@inheritDoc} */
-    protected List getNextKnuthElements(LayoutContext context, int alignment,
+    protected List<ListElement> getNextKnuthElements(LayoutContext context, int alignment,
             Position positionAtIPDChange, LayoutManager restartAtLM) {
-        List contentList = null;
+        List<ListElement> contentList = null;
 
         do {
             contentList = childFLM.getNextKnuthElements(context, alignment, positionAtIPDChange,
@@ -451,8 +451,10 @@ public class PageBreaker extends AbstractBreaker {
         int optimalPageCount = algRestart.findBreakingPoints(effectiveList,
                     newStartPos,
                     1, true, BreakingAlgorithm.ALL_BREAKS);
-        log.debug("restart: optimalPageCount= " + optimalPageCount
-                + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
+        if (algRestart.getPageBreaks() != null) {
+            log.debug("restart: optimalPageCount= " + optimalPageCount
+                    + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
+        }
 
         boolean fitsOnePage
             = optimalPageCount <= pslm.getCurrentPV()
@@ -619,6 +621,7 @@ public class PageBreaker extends AbstractBreaker {
                         && pv.getCurrentSpan().getColumnCount() == 1);
 
             if (forceNewPageWithSpan) {
+                checkPagePositionOnly();
                 log.trace("Forcing new page with span");
                 curPage = pslm.makeNewPage(false);
                 curPage.getPageViewport().createSpan(true);
@@ -630,6 +633,7 @@ public class PageBreaker extends AbstractBreaker {
                         log.trace("Moving to next flow");
                         pv.getCurrentSpan().moveToNextFlow();
                     } else {
+                        checkPagePositionOnly();
                         log.trace("Making new page");
                         pslm.makeNewPage(false, emptyContent);
                     }
@@ -641,14 +645,22 @@ public class PageBreaker extends AbstractBreaker {
         }
     }
 
+    private void checkPagePositionOnly() {
+        if (pslm.getCurrentPage().isPagePositionOnly && !pslm.fobj.getUserAgent().isLegacySkipPagePositionOnly()) {
+            throw new PagePositionOnlyException();
+        }
+    }
+
     private void handleBreakBeforeFollowingPage(int breakVal) {
         log.debug("handling break-before after page " + pslm.getCurrentPageNum() + " breakVal="
                 + getBreakClassName(breakVal));
         if (needBlankPageBeforeNew(breakVal)) {
+            checkPagePositionOnly();
             log.trace("Inserting blank page");
             /* curPage = */pslm.makeNewPage(true);
         }
         if (needNewPage(breakVal)) {
+            checkPagePositionOnly();
             log.trace("Making new page");
             /* curPage = */pslm.makeNewPage(false);
         }
@@ -924,8 +936,8 @@ public class PageBreaker extends AbstractBreaker {
         pageBreakHandled = true;
     }
 
-    public void holdFootnotes(List fl, List ll, int tfl, int ifl, boolean fp, boolean nf, int fnfi, int fli,
-            int fei, MinOptMax fsl, int pfli, int pfei) {
+    public void holdFootnotes(List fl, List<Integer> ll, int tfl, int ifl, boolean fp, boolean nf, int fnfi, int fli,
+                              int fei, MinOptMax fsl, int pfli, int pfei) {
         relayedFootnotesList = fl;
         relayedLengthList = ll;
         relayedTotalFootnotesLength = tfl;
