@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LineLayoutManager.java 1862365 2019-07-01 10:18:11Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -108,7 +108,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
      * Each value holds the start and end indexes into a List of
      * inline break positions.
      */
-    static class LineBreakPosition extends LeafPosition {
+    public static class LineBreakPosition extends LeafPosition {
         private final int parIndex; // index of the Paragraph this Position refers to
         private final int startIndex; //index of the first element this Position refers to
         private final int availableShrink;
@@ -607,7 +607,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
 
     /** {@inheritDoc} */
     @Override
-    public List getNextKnuthElements(LayoutContext context, int alignment) {
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
         if (alignmentContext == null) {
             FontInfo fi = fobj.getFOEventHandler().getFontInfo();
             FontTriplet[] fontkeys = fobj.getCommonFont().getFontState(fi);
@@ -649,16 +649,29 @@ public class LineLayoutManager extends InlineStackingLayoutManager
      * @return the list of KnuthElements
      * @see LayoutManager#getNextKnuthElements(LayoutContext,int)
      */
-    public List getNextKnuthElements(LayoutContext context, int alignment,
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment,
             LeafPosition restartPosition) {
-        log.trace("Restarting line breaking from index " + restartPosition.getIndex());
-        int parIndex = restartPosition.getLeafPos();
+        int parIndex = 0;
+        int restartPositionIdx = 0;
+        if (restartPosition != null) {
+            log.trace("Restarting line breaking from index " + restartPosition.getIndex());
+            parIndex = restartPosition.getLeafPos();
+            restartPositionIdx = restartPosition.getIndex();
+        }
+
+        for (int i = 0; i < parIndex; i++) {
+            if (knuthParagraphs.size() > 1) {
+                knuthParagraphs.remove(0);
+            }
+        }
+        parIndex = 0;
+
         KnuthSequence paragraph = knuthParagraphs.get(parIndex);
         if (paragraph instanceof Paragraph) {
             ((Paragraph) paragraph).ignoreAtStart = 0;
             isFirstInBlock = false;
         }
-        paragraph.subList(0, restartPosition.getIndex() + 1).clear();
+        paragraph.subList(0, restartPositionIdx + 1).clear();
         Iterator<KnuthElement> iter = paragraph.iterator();
         while (iter.hasNext() && !iter.next().isBox()) {
             iter.remove();
@@ -695,7 +708,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
 
         InlineLevelLayoutManager curLM;
         while ((curLM = (InlineLevelLayoutManager) getChildLM()) != null) {
-            List inlineElements = curLM.getNextKnuthElements(inlineLC, effectiveAlignment);
+            List<KnuthSequence> inlineElements = curLM.getNextKnuthElements(inlineLC, effectiveAlignment);
             if (inlineElements == null || inlineElements.size() == 0) {
                 /* curLM.getNextKnuthElements() returned null or an empty list;
                  * this can happen if there is nothing more to layout,
@@ -704,7 +717,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
             }
 
             if (lastPar != null) {
-                KnuthSequence firstSeq = (KnuthSequence) inlineElements.get(0);
+                KnuthSequence firstSeq = inlineElements.get(0);
 
                 // finish last paragraph before a new block sequence
                 if (!firstSeq.isInlineSequence()) {
@@ -881,12 +894,14 @@ public class LineLayoutManager extends InlineStackingLayoutManager
 
             // now try something different
             log.debug("Hyphenation possible? " + canHyphenate);
+            boolean simpleLineBreaking = fobj.getUserAgent().isSimpleLineBreaking();
+
             // Note: if allowedBreaks is guaranteed to be unchanged by alg.findBreakingPoints(),
             // the below check can be simplified to 'if (canHyphenate) ...'
             if (canHyphenate && allowedBreaks != BreakingAlgorithm.ONLY_FORCED_BREAKS) {
                 // consider every hyphenation point as a legal break
                 allowedBreaks = BreakingAlgorithm.ALL_BREAKS;
-            } else {
+            } else if (!simpleLineBreaking) {
                 // try with a higher threshold
                 maxAdjustment = 5;
             }
@@ -899,7 +914,9 @@ public class LineLayoutManager extends InlineStackingLayoutManager
                     log.debug("No set of breaking points found with maxAdjustment = "
                             + maxAdjustment + (canHyphenate ? " and hyphenation" : ""));
                 }
-                maxAdjustment = 20;
+                if (!simpleLineBreaking) {
+                    maxAdjustment = 20;
+                }
                 alg.findBreakingPoints(currPar, maxAdjustment, true, allowedBreaks);
             }
 
@@ -918,7 +935,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
      */
     private List<ListElement> postProcessLineBreaks(int alignment, LayoutContext context) {
 
-        List<ListElement> returnList = new LinkedList<ListElement>();
+        List<ListElement> returnList = new LinkedList<>();
 
         int endIndex = -1;
         for (int p = 0; p < knuthParagraphs.size(); p++) {
@@ -1205,14 +1222,14 @@ public class LineLayoutManager extends InlineStackingLayoutManager
 
     /** {@inheritDoc} */
     @Override
-    public List getChangedKnuthElements(List oldList, int alignment, int depth) {
+    public List<ListElement> getChangedKnuthElements(List<ListElement> oldList, int alignment, int depth) {
         return getChangedKnuthElements(oldList, alignment);
     }
 
     /** {@inheritDoc} */
     @Override
-    public List getChangedKnuthElements(List oldList, int alignment) {
-        List<KnuthElement> returnList = new LinkedList<KnuthElement>();
+    public List<ListElement> getChangedKnuthElements(List<ListElement> oldList, int alignment) {
+        List<ListElement> returnList = new LinkedList<>();
         for (int p = 0; p < knuthParagraphs.size(); p++) {
             LineLayoutPossibilities llPoss = lineLayoutsList[p];
             //log.debug("demerits of the chosen layout: " + llPoss.getChosenDemerits());
@@ -1266,7 +1283,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
         // hyphenate every word
         ListIterator currParIterator = currPar.listIterator(currPar.ignoreAtStart);
         // list of TLM involved in hyphenation
-        List updateList = new LinkedList();
+        List<Update> updateList = new LinkedList<>();
         KnuthElement firstElement;
         KnuthElement nextElement;
         // current InlineLevelLayoutManager
@@ -1351,7 +1368,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
         processUpdates(currPar, updateList);
     }
 
-    private void processUpdates(Paragraph par, List updateList) {
+    private void processUpdates(Paragraph par, List<Update> updateList) {
         // create iterator for the updateList
         ListIterator updateListIterator = updateList.listIterator();
         Update currUpdate;
@@ -1379,7 +1396,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
                 .applyChanges(par.subList(fromIndex + elementsAdded,
                                               toIndex + elementsAdded))) {
                 // insert the new KnuthElements
-                List newElements = currUpdate.inlineLM.getChangedKnuthElements(
+                List<ListElement> newElements = currUpdate.inlineLM.getChangedKnuthElements(
                     par.subList(fromIndex + elementsAdded,
                                      toIndex + elementsAdded),
                      /*flaggedPenalty,*/ effectiveAlignment);
@@ -1610,7 +1627,7 @@ public class LineLayoutManager extends InlineStackingLayoutManager
          * The positionList must contain one area-generating position,
          * which creates one line area.
          */
-        List positionList = new ArrayList(1);
+        List<Position> positionList = new ArrayList<>(1);
         Position innerPosition = pos.getPosition();
         positionList.add(innerPosition);
 
