@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ListItemLayoutManager.java 1884905 2020-12-29 12:58:04Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr.list;
 
@@ -44,6 +44,7 @@ import org.apache.fop.layoutmgr.ElementListObserver;
 import org.apache.fop.layoutmgr.ElementListUtils;
 import org.apache.fop.layoutmgr.FloatContentLayoutManager;
 import org.apache.fop.layoutmgr.FootenoteUtil;
+import org.apache.fop.layoutmgr.FootnoteBodyLayoutManager;
 import org.apache.fop.layoutmgr.Keep;
 import org.apache.fop.layoutmgr.KnuthBlockBox;
 import org.apache.fop.layoutmgr.KnuthElement;
@@ -211,7 +212,7 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
     }
 
     /** {@inheritDoc} */
-    public List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
             Position restartPosition, LayoutManager restartAtLM) {
         referenceIPD = context.getRefIPD();
         LayoutContext childLC;
@@ -422,7 +423,7 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
             // collect footnote information
             // TODO this should really not be done like this. ListItemLM should remain as
             // footnote-agnostic as possible
-            LinkedList<LayoutManager> footnoteList = new LinkedList<LayoutManager>();
+            LinkedList<FootnoteBodyLayoutManager> footnoteList = new LinkedList<>();
             for (int i = 0; i < elementLists.length; i++) {
                 footnoteList.addAll(FootenoteUtil.getFootnotes(elementLists[i], start[i], end[i]));
             }
@@ -436,6 +437,15 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
             addedBoxHeight += boxHeight;
             ListItemPosition stepPosition = new ListItemPosition(this, start[0], end[0], start[1], end[1]);
             stepPosition.setOriginalLabelPosition(originalLabelPosition);
+            if (originalBodyPosition == null || originalBodyPosition.getLM() instanceof ListItemContentLayoutManager) {
+                // Happens when ListItem has multiple blocks and a block (that's not the last block) ends at the same
+                // page height as a IPD change (e.g. FOP-3098). originalBodyPosition (reset) position needs to be a
+                // Block so that BlockStackingLayoutManager can stack it. Lookahead to find next Block.
+                Position block = extractBlock(elementLists[1], end[1] + 1);
+                if (block != null) {
+                    originalBodyPosition = block;
+                }
+            }
             stepPosition.setOriginalBodyPosition(originalBodyPosition);
 
             if (floats.isEmpty()) {
@@ -467,7 +477,39 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
         return returnList;
     }
 
-    private boolean shouldWeAvoidBreak(List returnList, LayoutManager lm) {
+    /**
+     * Extracts a block Position from a ListElement at a given index in a list of ListItem body elements.
+     * @param bodyElements The ListItem body elements.
+     * @param index        The index of the ListElement containing the block.
+     * @return             The required block Position as a LeafPosition or null on failure.
+     */
+    private Position extractBlock(List<ListElement> bodyElements, int index) {
+        ListElement listElement;
+        Position position = null;
+        Position retval = null;
+        do {
+            if (bodyElements == null || index >= bodyElements.size()) {
+                break;
+            }
+            listElement = bodyElements.get(index);
+            if (listElement != null
+                    && listElement.getLayoutManager() instanceof ListItemContentLayoutManager) {
+                position = listElement.getPosition();
+                if (position != null
+                        && position.getLM() instanceof ListItemContentLayoutManager) {
+                    position = position.getPosition();
+                    if (position != null
+                            && position.getPosition() != null
+                            && position.getLM() instanceof BlockLayoutManager) {
+                        retval = new LeafPosition(position.getPosition().getLM(), 0);
+                    }
+                }
+            }
+        } while (false);
+        return retval;
+    }
+
+    private boolean shouldWeAvoidBreak(List<ListElement> returnList, LayoutManager lm) {
         if (isChangingIPD(lm)) {
             if (lm instanceof BlockLayoutManager) {
                 return true;
@@ -492,6 +534,9 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
     private boolean isChangingIPD(LayoutManager lm) {
         PageProvider pageProvider = lm.getPSLM().getPageProvider();
         int currentIPD = pageProvider.getCurrentIPD();
+        if (currentIPD == -1) {
+            return false;
+        }
         int nextIPD = pageProvider.getNextIPD();
         return nextIPD != currentIPD;
     }
@@ -570,7 +615,7 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
 
     /** {@inheritDoc} */
     @Override
-    public List getChangedKnuthElements(List oldList, int alignment) {
+    public List<ListElement> getChangedKnuthElements(List oldList, int alignment) {
         // label
         labelList = label.getChangedKnuthElements(labelList, alignment);
 
@@ -592,11 +637,11 @@ public class ListItemLayoutManager extends SpacedBorderedPaddedBlockLayoutManage
             }
         }
 
-        List returnedList = body.getChangedKnuthElements(oldList, alignment);
+        List<ListElement> returnedList = body.getChangedKnuthElements(oldList, alignment);
         // "wrap" the Position inside each element
-        List tempList = returnedList;
+        List<ListElement> tempList = returnedList;
         KnuthElement tempElement;
-        returnedList = new LinkedList();
+        returnedList = new LinkedList<>();
         for (Object aTempList : tempList) {
             tempElement = (KnuthElement) aTempList;
             tempElement.setPosition(new NonLeafPosition(this, tempElement.getPosition()));
