@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableCellLayoutManager.java 1835810 2018-07-13 10:29:57Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -54,6 +54,7 @@ import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.KnuthPenalty;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.LayoutManager;
+import org.apache.fop.layoutmgr.ListElement;
 import org.apache.fop.layoutmgr.LocalBreaker;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
@@ -168,16 +169,16 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
     /**
      * {@inheritDoc}
      */
-    public List getNextKnuthElements(LayoutContext context, int alignment) {
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
         MinOptMax stackLimit = context.getStackLimitBP();
 
         referenceIPD = context.getRefIPD();
         cellIPD = referenceIPD;
         cellIPD -= getIPIndents();
 
-        List returnedList;
-        List contentList = new LinkedList();
-        List returnList = new LinkedList();
+        List<ListElement> returnedList;
+        List<ListElement> contentList = new LinkedList<>();
+        List<ListElement> returnList = new LinkedList<>();
 
         LayoutManager curLM; // currently active LM
         LayoutManager prevLM = null; // previously active LM
@@ -217,7 +218,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
         }
         primaryGridUnit.setKeepWithNext(context.getKeepWithNextPending());
 
-        returnedList = new LinkedList();
+        returnedList = new LinkedList<>();
         if (!contentList.isEmpty()) {
             wrapPositionElements(contentList, returnList);
         } else {
@@ -229,7 +230,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
         }
         //Space resolution
         SpaceResolver.resolveElementList(returnList);
-        if (((KnuthElement) returnList.get(0)).isForcedBreak()) {
+        if (returnList.get(0).isForcedBreak()) {
             primaryGridUnit.setBreakBefore(((KnuthPenalty) returnList.get(0)).getBreakClass());
             returnList.remove(0);
             assert !returnList.isEmpty();
@@ -703,7 +704,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
      * @param childArea the child to add to the cell
      */
     public void addChildArea(Area childArea) {
-        if (curBlockArea != null) {
+        if (curBlockArea != null && childArea instanceof Block) {
             curBlockArea.addBlock((Block) childArea);
         }
     }
