@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableLayoutManager.java 1884907 2020-12-29 13:33:34Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -34,6 +34,7 @@ import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
+import org.apache.fop.fo.flow.AbstractGraphics;
 import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.flow.Markers;
 import org.apache.fop.fo.flow.RetrieveTableMarker;
@@ -86,7 +87,7 @@ public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
     private int halfBorderSeparationIPD;
 
     /** See {@link TableLayoutManager#registerColumnBackgroundArea(TableColumn, Block, int)}. */
-    private List columnBackgroundAreas;
+    private List<ColumnBackgroundInfo> columnBackgroundAreas;
 
     private Position auxiliaryPosition;
 
@@ -202,9 +203,9 @@ public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
     }
 
     /** {@inheritDoc} */
-    public List getNextKnuthElements(LayoutContext context, int alignment) {
+    public List<ListElement> getNextKnuthElements(LayoutContext context, int alignment) {
 
-        List returnList = new LinkedList();
+        List<ListElement> returnList = new LinkedList<>();
 
         /*
          * Compute the IPD and adjust it if necessary (overconstrained)
@@ -243,7 +244,9 @@ public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
          */
         if (tableUnit == 0.0) {
             tableUnit = columns.computeTableUnit(this);
-            tableUnit = Math.max(tableUnit, oldTableUnit);
+            if (oldTableUnit > tableUnit && supportResize(fobj)) {
+                tableUnit = oldTableUnit;
+            }
         }
 
         if (!firstVisibleMarkServed) {
@@ -314,6 +317,20 @@ public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
         return returnList;
     }
 
+    private boolean supportResize(FONode node) {
+        if (node instanceof AbstractGraphics || getPSLM().getPageProvider().getCurrentColumnCount() == 1) {
+            return false;
+        }
+        FONode.FONodeIterator iterator = node.getChildNodes();
+        while (iterator != null && iterator.hasNext()) {
+            FONode x = iterator.next();
+            if (!supportResize(x)) {
+                return false;
+            }
+        }
+        return true;
+    }
+
     /** {@inheritDoc} */
     public Position getAuxiliaryPosition() {
         /*
@@ -346,7 +363,7 @@ public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
     void registerColumnBackgroundArea(TableColumn column, Block backgroundArea, int xShift) {
         addBackgroundArea(backgroundArea);
         if (columnBackgroundAreas == null) {
-            columnBackgroundAreas = new ArrayList();
+            columnBackgroundAreas = new ArrayList<>();
         }
         columnBackgroundAreas.add(new ColumnBackgroundInfo(column, backgroundArea, xShift));
     }
