@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocument.java 1866666 2019-09-09 09:08:12Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.pdf;
 
@@ -23,12 +23,14 @@ package org.apache.fop.pdf;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
+import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -36,6 +38,8 @@ import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.image.loader.util.SoftMapCache;
+
 import org.apache.fop.pdf.StandardStructureAttributes.Table.Scope;
 import org.apache.fop.pdf.xref.CrossReferenceStream;
 import org.apache.fop.pdf.xref.CrossReferenceTable;
@@ -131,6 +135,7 @@ public class PDFDocument {
 
     /* TODO: Should be modified (works only for image subtype) */
     private Map<String, PDFXObject> xObjectsMap = new HashMap<String, PDFXObject>();
+    private SoftMapCache xObjectsMapFast = new SoftMapCache(false);
 
     private Map<String, PDFFont> fontMap = new HashMap<String, PDFFont>();
 
@@ -172,6 +177,8 @@ public class PDFDocument {
 
     private boolean mergeFontsEnabled;
 
+    private boolean mergeFormFieldsEnabled;
+
     private boolean linearizationEnabled;
 
     private boolean formXObjectEnabled;
@@ -383,7 +390,6 @@ public class PDFDocument {
      * Adds the given element to the structure tree.
      */
     public void registerStructureElement(PDFStructElem structElem) {
-        assignObjectNumber(structElem);
         structureTreeElements.add(structElem);
     }
 
@@ -428,7 +434,7 @@ public class PDFDocument {
      * @param obj {@link PDFObject} to add
      * @return the added {@link PDFObject} added (with its object number set)
      */
-    <T extends PDFObject> T registerTrailerObject(T obj) {
+    public <T extends PDFObject> T registerTrailerObject(T obj) {
         assignObjectNumber(obj);
         addTrailerObject(obj);
         return obj;
@@ -473,6 +479,9 @@ public class PDFDocument {
      * @param obj {@link PDFObject} to add
      */
     public void addObject(PDFObject obj) {
+        if (obj instanceof PDFStructElem) {
+            return;
+        }
         if (obj == null) {
             throw new NullPointerException("obj must not be null");
         }
@@ -747,7 +756,7 @@ public class PDFDocument {
             poss = new PDFGState();
             poss.addValues(current);
             poss.addValues(avail);
-            if (poss.equals(wanted)) {
+            if (poss.contentEquals(wanted)) {
                 return avail;
             }
         }
@@ -800,7 +809,7 @@ public class PDFDocument {
      */
     @Deprecated
     public PDFImageXObject getImage(String key) {
-        return (PDFImageXObject)this.xObjectsMap.get(key);
+        return (PDFImageXObject)getXObject(key);
     }
 
     /**
@@ -810,7 +819,34 @@ public class PDFDocument {
      * @return the PDFXObject for the key if found
      */
     public PDFXObject getXObject(String key) {
-        return this.xObjectsMap.get(key);
+        Object xObj = xObjectsMapFast.get(key);
+        if (xObj != null) {
+            return (PDFXObject) xObj;
+        }
+        return xObjectsMap.get(toHashCode(key));
+    }
+
+    private void putXObject(String key, PDFXObject pdfxObject) {
+        xObjectsMapFast.clear();
+        xObjectsMapFast.put(key, pdfxObject);
+        xObjectsMap.put(toHashCode(key), pdfxObject);
+    }
+
+    private String toHashCode(String key) {
+        if (key.length() < 1024) {
+            return key;
+        }
+        try {
+            MessageDigest md = MessageDigest.getInstance("MD5");
+            byte[] thedigest = md.digest(key.getBytes("UTF-8"));
+            StringBuilder hex = new StringBuilder();
+            for (byte b : thedigest) {
+                hex.append(String.format("%02x", b));
+            }
+            return hex.toString();
+        } catch (NoSuchAlgorithmException | UnsupportedEncodingException e) {
+            throw new RuntimeException(e);
+        }
     }
 
     /**
@@ -859,7 +895,7 @@ public class PDFDocument {
     public PDFImageXObject addImage(PDFResourceContext res, PDFImage img) {
         // check if already created
         String key = img.getKey();
-        PDFImageXObject xObject = (PDFImageXObject)this.xObjectsMap.get(key);
+        PDFImageXObject xObject = (PDFImageXObject)getXObject(key);
         if (xObject != null) {
             if (res != null) {
                 res.addXObject(xObject);
@@ -876,7 +912,7 @@ public class PDFDocument {
         if (res != null) {
             res.addXObject(xObject);
         }
-        this.xObjectsMap.put(key, xObject);
+        putXObject(key, xObject);
         return xObject;
     }
 
@@ -899,7 +935,7 @@ public class PDFDocument {
         String key) {
 
         // check if already created
-        PDFFormXObject xObject = (PDFFormXObject)xObjectsMap.get(key);
+        PDFFormXObject xObject = (PDFFormXObject)getXObject(key);
         if (xObject != null) {
             if (res != null) {
                 res.addXObject(xObject);
@@ -916,7 +952,7 @@ public class PDFDocument {
         if (res != null) {
             res.addXObject(xObject);
         }
-        this.xObjectsMap.put(key, xObject);
+        putXObject(key, xObject);
         return xObject;
     }
 
@@ -1116,6 +1152,14 @@ public class PDFDocument {
                 ? new CompressedTrailerOutputHelper()
                 : new UncompressedTrailerOutputHelper();
         if (structureTreeElements != null) {
+            Iterator<PDFStructElem> structElemIterator = structureTreeElements.iterator();
+            while (structElemIterator.hasNext()) {
+                PDFStructElem structElem = structElemIterator.next();
+                if (!structElem.hasObjectNumber()) {
+                    structElemIterator.remove();
+                    structElem.parentElement.kids.remove(structElem);
+                }
+            }
             trailerOutputHelper.outputStructureTreeElements(stream);
         }
         streamIndirectObjects(trailerObjects, stream);
@@ -1156,6 +1200,14 @@ public class PDFDocument {
         }
     }
 
+    public boolean isMergeFormFieldsEnabled() {
+        return mergeFormFieldsEnabled;
+    }
+
+    public void setMergeFormFieldsEnabled(boolean mergeFormFieldsEnabled) {
+        this.mergeFormFieldsEnabled = mergeFormFieldsEnabled;
+    }
+
     private interface TrailerOutputHelper {
 
         void outputStructureTreeElements(OutputStream stream) throws IOException;
