@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPDocumentHandler.java 1866691 2019-09-09 13:20:08Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.afp;
 
@@ -164,7 +164,7 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
                     getPaintingState(), getDataStream(), getResourceManager());
         } else {
             return new AFPRectanglePainter(
-                    getPaintingState(), getDataStream());
+                    getPaintingState(), getDataStream(), getResourceManager());
         }
     }
 
@@ -215,7 +215,9 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
     /** {@inheritDoc} */
     public void startPageSequence(String id) throws IFException {
         try {
-            dataStream.startPageGroup();
+            if (!"false".equals(getContext().getForeignAttribute(AFPElementMapping.PAGE_GROUP))) {
+                dataStream.startPageGroup();
+            }
         } catch (IOException ioe) {
             throw new IFException("I/O error in startPageSequence()", ioe);
         }
@@ -543,6 +545,10 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
         paintingState.setFS45(fs45);
     }
 
+    public void setMaskEnabled(boolean maskEnabled) {
+        paintingState.setMaskEnabled(maskEnabled);
+    }
+
     /** {@inheritDoc} */
     public boolean getWrapPSeg() {
         return  paintingState.getWrapPSeg();
