@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPPainter.java 1884462 2020-12-15 13:28:47Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.afp;
 
@@ -70,7 +70,6 @@ import org.apache.fop.afp.ptoca.PtocaProducer;
 import org.apache.fop.afp.util.AFPResourceAccessor;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontTriplet;
-import org.apache.fop.fonts.FontType;
 import org.apache.fop.fonts.Typeface;
 import org.apache.fop.render.ImageHandlerUtil;
 import org.apache.fop.render.RenderingContext;
@@ -1025,12 +1024,13 @@ public class AFPPainter extends AbstractIFPainter<AFPDocumentHandler> {
                 builder.setVariableSpaceCharacterIncrement(varSpaceCharacterIncrement);
 
                 boolean fixedSpaceMode = false;
-                int ttPos = p.x;
-
+                double ttPos = p.x;
+                boolean positionByChar = afpFont instanceof AFPFontConfig.AFPTrueTypeFont
+                        && ((AFPFontConfig.AFPTrueTypeFont) afpFont).isPositionByChar();
                 for (int i = 0; i < l; i++) {
                     char orgChar = text.charAt(i);
                     float glyphAdjust = 0;
-                    if (afpFont.getFontType() == FontType.TRUETYPE) {
+                    if (positionByChar) {
                         flushText(builder, sb, charSet);
                         fixedSpaceMode = true;
                         int charWidth = font.getCharWidth(orgChar);
@@ -1064,10 +1064,10 @@ public class AFPPainter extends AbstractIFPainter<AFPDocumentHandler> {
                         glyphAdjust += dx[i + 1];
                     }
 
-                    if (afpFont.getFontType() == FontType.TRUETYPE) {
+                    if (positionByChar) {
                         flushText(builder, sb, charSet);
-                        ttPos += Math.round(unitConv.mpt2units(glyphAdjust));
-                        builder.absoluteMoveInline(ttPos);
+                        ttPos += unitConv.mpt2units(glyphAdjust);
+                        builder.absoluteMoveInline((int) Math.round(ttPos));
                     } else if (glyphAdjust != 0) {
                         flushText(builder, sb, charSet);
                         int increment = Math.round(unitConv.mpt2units(glyphAdjust));
