@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IFSerializer.java 1875656 2020-03-25 16:36:47Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.intermediate;
 
@@ -914,8 +914,11 @@ implements IFConstants, IFPainter, IFDocumentNavigationHandler {
         atts.addAttribute("", "rect", "rect",
                 XMLConstants.CDATA, IFUtil.toString(link.getTargetRect()));
         if (getUserAgent().isAccessibilityEnabled()) {
-            addStructRefAttribute(atts,
-                    ((IFStructureTreeElement) link.getAction().getStructureTreeElement()).getId());
+            IFStructureTreeElement structureTreeElement =
+                    (IFStructureTreeElement) link.getAction().getStructureTreeElement();
+            if (structureTreeElement != null) {
+                addStructRefAttribute(atts, structureTreeElement.getId());
+            }
         }
         try {
             handler.startElement(DocumentNavigationExtensionConstants.LINK, atts);
