@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFPainter.java 1885366 2021-01-11 15:00:20Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.pdf;
 
@@ -245,6 +245,7 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
         PDFRenderingContext pdfContext = new PDFRenderingContext(
                 getUserAgent(), generator, getDocumentHandler().getCurrentPage(), getFontInfo());
         pdfContext.setMarkedContentInfo(imageMCI);
+        pdfContext.setUsedFieldNames(getDocumentHandler().getUsedFieldNames());
         pdfContext.setPageNumbers(getDocumentHandler().getPageNumbers());
         pdfContext.setPdfLogicalStructureHandler(logicalStructureHandler);
         pdfContext.setCurrentSessionStructElem((PDFStructElem) getContext().getStructureTreeElement());
@@ -435,8 +436,12 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
         String fontKey = getFontInfo().getInternalFontKey(triplet);
         Typeface typeface = getTypeface(fontKey);
         if (typeface instanceof MultiByteFont && ((MultiByteFont) typeface).hasSVG()) {
-            drawSVGText((MultiByteFont) typeface, triplet, x, y, text, state);
-        } else if ((dp == null) || IFUtil.isDPOnlyDX(dp)) {
+            boolean success = drawSVGText((MultiByteFont) typeface, triplet, x, y, text, state);
+            if (success) {
+                return;
+            }
+        }
+        if ((dp == null) || IFUtil.isDPOnlyDX(dp)) {
             drawTextWithDX(x, y, text, triplet, letterSpacing,
                              wordSpacing, IFUtil.convertDPToDX(dp));
         } else {
@@ -469,17 +474,7 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
         PDFTextUtil textutil = generator.getTextUtil();
         textutil.updateTf(fontKey, fontSize, tf.isMultiByte(), tf.isCID());
 
-        double shear = 0;
-        boolean simulateStyle = tf instanceof CustomFont && ((CustomFont) tf).getSimulateStyle();
-        if (simulateStyle) {
-            if (triplet.getWeight() == 700) {
-                generator.add("q\n");
-                generator.add("2 Tr 0.31543 w\n");
-            }
-            if (triplet.getStyle().equals("italic")) {
-                shear = 0.3333;
-            }
-        }
+        double shear = startSimulateStyle(tf, triplet);
 
         generator.updateCharacterSpacing(letterSpacing / 1000f);
 
@@ -532,8 +527,29 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
 
         }
         textutil.writeTJ();
+        generator.resetCharacterSpacing();
+        endSimulateStyle(tf, triplet);
+    }
+
+    private double startSimulateStyle(Typeface tf, FontTriplet triplet) {
+        double shear = 0;
+        boolean simulateStyle = tf instanceof CustomFont && ((CustomFont) tf).getSimulateStyle();
+        if (simulateStyle) {
+            if (triplet.getWeight() == 700) {
+                generator.updateColor(state.getTextColor(), false, null);
+                generator.add("2 Tr 0.31543 w\n");
+            }
+            if (triplet.getStyle().equals("italic")) {
+                shear = 0.3333;
+            }
+        }
+        return shear;
+    }
+
+    private void endSimulateStyle(Typeface tf, FontTriplet triplet) {
+        boolean simulateStyle = tf instanceof CustomFont && ((CustomFont) tf).getSimulateStyle();
         if (simulateStyle && triplet.getWeight() == 700) {
-            generator.add("Q\n");
+            generator.add("0 Tr\n");
         }
     }
 
@@ -557,25 +573,20 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
             double      yoLast          = 0f;
             double      wox             = wordSpacing;
 
-            // FOP-2810
-            boolean simulateStyle = tf instanceof CustomFont && ((CustomFont) tf).getSimulateStyle();
-            double shear = 0;
-
-            if (simulateStyle) {
-                if (triplet.getWeight() == 700) {
-                    generator.add("q\n");
-                    generator.add("2 Tr 0.31543 w\n");
-                }
-                if (triplet.getStyle().equals("italic")) {
-                    shear = 0.3333;
-                }
-            }
+            double shear = startSimulateStyle(tf, triplet);
 
             tu.writeTextMatrix(new AffineTransform(1, 0, shear, -1, x / 1000f, y / 1000f));
             tu.updateTf(fk, fsPoints, tf.isMultiByte(), true);
             generator.updateCharacterSpacing(letterSpacing / 1000f);
             for (int i = 0, n = text.length(); i < n; i++) {
-                char    ch              = text.charAt(i);
+                int ch = text.charAt(i);
+                int mp;
+                if (CharUtilities.containsSurrogatePairAt(text, i)) {
+                    ch = Character.toCodePoint((char) ch, text.charAt(++i));
+                    mp = f.mapCodePoint(ch);
+                } else {
+                    mp = f.mapChar((char)ch);
+                }
                 int[]   pa              = ((i >= dp.length) || (dp[i] == null)) ? paZero : dp[i];
                 double  xo              = xc + pa[0];
                 double  yo              = yc + pa[1];
@@ -584,16 +595,18 @@ public class PDFPainter extends AbstractIFPainter<PDFDocumentHandler> {
                 double  xd              = (xo - xoLast) / 1000f;
                 double  yd              = (yo - yoLast) / 1000f;
                 tu.writeTd(xd, yd);
-                tu.writeTj(f.mapChar(ch), tf.isMultiByte(), true);
+                tu.writeTj(mp, tf.isMultiByte(), true);
                 xc += xa + pa[2];
                 yc += ya + pa[3];
                 xoLast = xo;
                 yoLast = yo;
             }
+            generator.resetCharacterSpacing();
+            endSimulateStyle(tf, triplet);
         }
     }
 
-    private double maybeWordOffsetX(double wox, char ch, Direction dir) {
+    private double maybeWordOffsetX(double wox, int ch, Direction dir) {
         if ((wox != 0)
              && CharUtilities.isAdjustableSpace(ch)
              && ((dir == null) || dir.isHorizontal())) {
