@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFRenderingUtil.java 1866666 2019-09-09 09:08:12Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.pdf;
 
@@ -47,6 +47,7 @@ import org.apache.xmlgraphics.xmp.schemas.XMPBasicSchema;
 import org.apache.fop.accessibility.Accessibility;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.io.InternalResourceResolver;
+import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.extensions.xmp.XMPMetadata;
 import org.apache.fop.pdf.PDFAMode;
 import org.apache.fop.pdf.PDFArray;
@@ -73,6 +74,7 @@ import org.apache.fop.pdf.PDFPage;
 import org.apache.fop.pdf.PDFPageLabels;
 import org.apache.fop.pdf.PDFReference;
 import org.apache.fop.pdf.PDFSetOCGStateAction;
+import org.apache.fop.pdf.PDFSignParams;
 import org.apache.fop.pdf.PDFTransitionAction;
 import org.apache.fop.pdf.PDFXMode;
 import org.apache.fop.pdf.Version;
@@ -136,7 +138,7 @@ class PDFRenderingUtil {
         }
     }
 
-    private static  PDFRendererOptionsConfig createFromUserAgent(FOUserAgent userAgent) {
+    protected static PDFRendererOptionsConfig createFromUserAgent(FOUserAgent userAgent) {
         Map<PDFRendererOption, Object> properties
                 = new EnumMap<PDFRendererOption, Object>(PDFRendererOption.class);
         for (PDFRendererOption option : PDFRendererOption.values()) {
@@ -467,6 +469,7 @@ class PDFRenderingUtil {
             assert extension instanceof PDFPageExtension;
             if (((PDFPageExtension) extension).matchesPageNumber(currentPage.getPageIndex() + 1)) {
                 augmentDictionary(currentPage, extension);
+                renderExtension(currentPage, extension.getExtension());
             }
         } else if (type == PDFDictionaryType.Info) {
             PDFInfo info = pdfDoc.getInfo();
@@ -490,6 +493,15 @@ class PDFRenderingUtil {
         }
     }
 
+    private void renderExtension(PDFPage currentPage, ExtensionAttachment extension) {
+        if (extension instanceof XMPMetadata) {
+            XMPMetadata metadata = (XMPMetadata) extension;
+            Metadata docXMP = metadata.getMetadata();
+            PDFMetadata pdfMetadata = pdfDoc.getFactory().makeMetadata(docXMP, metadata.isReadOnly());
+            currentPage.setMetadata(pdfMetadata);
+        }
+    }
+
     private PDFDictionary augmentDictionary(PDFDictionary dictionary, PDFDictionaryExtension extension) {
         for (PDFCollectionEntryExtension entry : extension.getEntries()) {
             if (entry instanceof PDFDictionaryExtension) {
@@ -592,6 +604,7 @@ class PDFRenderingUtil {
                     = VersionController.getFixedVersionController(maxPDFVersion);
             this.pdfDoc = new PDFDocument(producer, controller);
         }
+        pdfDoc.getFactory().setEventBroadcaster(userAgent.getEventBroadcaster());
         updateInfo();
         updatePDFProfiles();
         pdfDoc.setFilterMap(rendererConfig.getFilterMap());
@@ -619,12 +632,17 @@ class PDFRenderingUtil {
 
         this.pdfDoc.enableAccessibility(userAgent.isAccessibilityEnabled());
         pdfDoc.setMergeFontsEnabled(rendererConfig.getMergeFontsEnabled());
+        pdfDoc.setMergeFormFieldsEnabled(rendererConfig.getMergeFormFieldsEnabled());
         pdfDoc.setLinearizationEnabled(rendererConfig.getLinearizationEnabled());
         pdfDoc.setFormXObjectEnabled(rendererConfig.getFormXObjectEnabled());
 
         return this.pdfDoc;
     }
 
+    public PDFSignParams getSignParams() {
+        return rendererConfig.getSignParams();
+    }
+
     /**
      * Generates a page label in the PDF document.
      * @param pageIndex the index of the page
