@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ResourceHandler.java 1809628 2017-09-25 13:42:23Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.ps;
 
@@ -327,6 +327,10 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
                     userAgent.getEventBroadcaster());
             eventProducer.imageError(resTracker, (info != null ? info.toString() : uri),
                     ie, null);
+        } catch (Exception e) {
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    userAgent.getEventBroadcaster());
+            eventProducer.imageWritingError(this, e);
         }
     }
 
