@@ -15,13 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: LocatorFormatter.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id$ */
 
 package org.apache.fop.util.text;
 
 import org.xml.sax.Locator;
 
-import org.apache.fop.util.text.AdvancedMessageFormat.ObjectFormatter;
+import org.apache.fop.utils.text.AdvancedMessageFormat.ObjectFormatter;
 
 /**
  * Object formatter for the SAX Locator object.
