@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFactoryTestCase.java 1885366 2021-01-11 15:00:20Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.pdf;
 
@@ -30,6 +30,7 @@ import java.net.URI;
 
 import org.junit.Test;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
 import org.apache.xmlgraphics.io.ResourceResolver;
@@ -45,6 +46,7 @@ import org.apache.fop.fonts.FontType;
 import org.apache.fop.fonts.FontUris;
 import org.apache.fop.fonts.MultiByteFont;
 import org.apache.fop.fonts.NamedCharacter;
+import org.apache.fop.fonts.SingleByteEncoding;
 import org.apache.fop.fonts.SingleByteFont;
 import org.apache.fop.fonts.truetype.OFFontLoader;
 
@@ -109,6 +111,35 @@ public class PDFFactoryTestCase {
         assertEquals(sb.getAdditionalEncodingCount(), 1);
     }
 
+    @Test
+    public void testDifferencesStartValue() throws IOException {
+        PDFDocument doc = new PDFDocument("");
+        PDFFactory pdfFactory = new PDFFactory(doc);
+        TestSingleByteFont sb = new TestSingleByteFont(null);
+        sb.setFlags(0);
+        sb.setMapping(new SingleByteEncoding() {
+            public String getName() {
+                return "FOPPDFEncoding";
+            }
+
+            public char mapChar(char c) {
+                return 0;
+            }
+
+            public String[] getCharNameMap() {
+                return new String[]{"a"};
+            }
+
+            public char[] getUnicodeCharMap() {
+                return new char[]{1};
+            }
+        });
+        pdfFactory.makeFont("a", "a", "WinAnsiEncoding", sb, sb);
+        ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        doc.outputTrailer(bos);
+        assertTrue(bos.toString().contains("/Differences [1 /a]"));
+    }
+
     @Test
     public void testMakeTrueTypeFont() throws IOException {
         PDFDocument doc = new PDFDocument("");
@@ -134,6 +165,10 @@ public class PDFFactoryTestCase {
             File f = new File("test/resources/fonts/type1/a010013l.pfb");
             return new FileInputStream(f);
         }
+
+        public void setMapping(SingleByteEncoding mapping) {
+            this.mapping = mapping;
+        }
     }
 
     @Test
@@ -169,6 +204,25 @@ public class PDFFactoryTestCase {
         assertTrue(bos.toString().contains("/Subtype /Type1C"));
     }
 
+    @Test
+    public void testMakeOTFFontPDFA() throws IOException {
+        InternalResourceResolver rr =
+                ResourceResolverFactory.createDefaultInternalResourceResolver(new File(".").toURI());
+        PDFDocument doc = new PDFDocument("");
+        doc.getProfile().setPDFAMode(PDFAMode.PDFA_2A);
+        PDFFactory pdfFactory = new PDFFactory(doc);
+        URI uri = new File("test/resources/fonts/otf/SourceSansProBold.otf").toURI();
+        CustomFont sb = OFFontLoader.loadFont(new FontUris(uri, null),
+                null, true, EmbeddingMode.SUBSET, null, false, false, rr, false, false, true);
+        for (char c = 0; c < 512; c++) {
+            sb.mapChar(c);
+        }
+        pdfFactory.makeFont("a", "a", "WinAnsiEncoding", sb, sb);
+        PDFFont pdfFont = pdfFactory.getDocument().getFontMap().get("a_1");
+        PDFFontDescriptor fontDescriptor = (PDFFontDescriptor) pdfFont.get("FontDescriptor");
+        assertNull(fontDescriptor.getCIDSet());
+    }
+
     @Test
     public void testGetExternalAction() {
 
