@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BitmapProducerJava2D.java 1877383 2020-05-05 09:34:11Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.visual;
 
@@ -39,7 +39,7 @@ import org.apache.fop.apps.MimeConstants;
 import org.apache.fop.configuration.Configurable;
 import org.apache.fop.configuration.Configuration;
 import org.apache.fop.configuration.ConfigurationException;
-import org.apache.fop.util.DefaultErrorListener;
+import org.apache.fop.utils.DefaultErrorListener;
 
 /**
  * BitmapProducer implementation that uses the Java2DRenderer to create bitmaps.
